from __future__ import absolute_import, division, print_function, unicode_literals

import os

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes

ALGO_AES_GCM = 'aes-gcm'
DEFAULT_KEY_LENGTH = 32
DEFAULT_IV_LENGTH = 12


def open_aes_gcm(key_service, material):
    """
    Decrypts secrets stored by `seal_aes_ctr`.

    Allows for binary plaintext
    """
    key = key_service.decrypt(material['key'])
    return _open_aes_gcm(
        key,
        material['iv'],
        material['tag'],
        material['contents']
    )


def seal_aes_gcm(key_service, secret, key_length=DEFAULT_KEY_LENGTH, iv_length=DEFAULT_IV_LENGTH, binary_type=None):
    """
    Encrypts `secret` using the key service.

    You can decrypt with the companion method `open_aes_ctr`.
    """
    key, encoded_key = key_service.generate_key_data(key_length)
    iv, tag, ciphertext = _seal_aes_gcm(secret, key, iv_length)

    # Agh! a mighty break in abstraction
    # DynamoDB wont put `bytes` => `Binary` in python2
    # So we need to wrap it with a special type
    def Binary(value):
        if binary_type:
            return binary_type(value)
        return value
    return {
        'key': Binary(encoded_key),
        'contents': Binary(ciphertext),
        'iv': Binary(iv),
        'tag': Binary(tag),
        'algorithm': ALGO_AES_GCM,
    }


def _open_aes_gcm(key, iv, tag, ciphertext):
    decryptor = Cipher(
        algorithms.AES(key),
        modes.GCM(iv, tag),
        backend=default_backend()
    ).decryptor()
    return decryptor.update(ciphertext) + decryptor.finalize()


def _seal_aes_gcm(plaintext, key, iv_length):
    iv = os.urandom(int(iv_length))
    encryptor = Cipher(
        algorithms.AES(key),
        modes.GCM(iv),
        backend=default_backend()
    ).encryptor()
    ciphertext = encryptor.update(plaintext) + encryptor.finalize()
    return iv, encryptor.tag, ciphertext



from __future__ import absolute_import, division, print_function

import click
from .cli import main
from .dynamodb_storage_service import DynamoDbStorageService


@main.command('setup-dynamodb')
@click.option('--read-capacity', type=click.INT, default=1)
@click.option('--write-capacity', type=click.INT, default=1)
@click.pass_context
def cmd_setup(ctx, read_capacity, write_capacity):
    """
    Setup the credential table in AWS DynamoDB
    """
    storage_service = ctx.obj.storage_service
    if not isinstance(storage_service, DynamoDbStorageService):
        raise click.ClickException('Cannot setup unknown storage service')
    storage_service.setup(
        read_capacity, write_capacity
    )




import codecs
from os.path import dirname, join

from setuptools import setup, find_packages

here = dirname(__file__)


def read(*parts):
    return codecs.open(join(here, *parts), 'r').read()


def find_version(*file_paths):
    version = read(*file_paths).strip()
    if version == '':
        raise RuntimeError('No version found')
    return version


setup(
    name='credsmash',
    version=find_version('credsmash', 'VERSION'),

    maintainer="Nathan Muir",
    maintainer_email="ndmuir@gmail.com",

    url='https://github.com/3stack-software/credsmash',

    license='Apache2',
    description='A utility for managing secrets in the cloud using AWS KMS and DynamoDB',
    long_description=read('README.md'),
    long_description_content_type='text/markdown',

    packages=find_packages(exclude=('tests',)),

    package_data={
        'credsmash': ['VERSION']
    },

    python_requires=">=3.8",
    install_requires=[
        'cryptography',
        'boto3',
        'click',
        'six',
        'importlib_resources',
        'importlib_metadata'
    ],
    tests_require=[
        'pytest',
    ],
    extras_require={
        'yaml': ['PyYAML'],
        'templates': ['jinja2'],
        'documentation': [],
        'dev': ['PyYAML', 'jinja2', 'pytest']
    },
    entry_points={
        'console_scripts': [
            'credsmash = credsmash.cli:main'
        ],
        'credsmash.key_service': [
            'kms = credsmash.kms_key_service:KmsKeyService',
        ],
        'credsmash.storage_service': [
            'dynamodb = credsmash.dynamodb_storage_service:DynamoDbStorageService',
        ],
        'credsmash.cli': [
            'templates = credsmash.templates',
            'dynamodb = credsmash.cli_dynamodb'
        ]
    },

    classifiers=[
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'License :: OSI Approved :: Apache Software License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',
    ],
)
